using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using NTK2019.Web.Intefaces;
using NTK2019.Web.Models;
using NTK2019.Web.Options;

namespace NTK2019.Web.Services
{
    public class AiRestService : IAIRestService
    {
        private readonly MetricOptions metricOptions;

        public AiRestService(IOptions<MetricOptions> metricOptionsValue)
        {
            metricOptions = metricOptionsValue.Value;
        }

        private const string URL =
            "https://api.applicationinsights.io/v1/apps/{0}/{1}/{2}?{3}";

        public async Task<MetricResult> GetMetricAsync(string filter, string metricId, string timespan)
        {
            var client = new HttpClient();
            client.DefaultRequestHeaders.Accept.Add(
                new MediaTypeWithQualityHeaderValue("application/json"));
            client.DefaultRequestHeaders.Add("x-api-key", metricOptions.ApiKey);
            var req =
                $"https://api.applicationinsights.io/v1/apps/{metricOptions.AppId}/metrics/{metricId}?timespan={timespan}";

            var response = await client.GetAsync(req);
            var metricResult = new MetricResult {MetricId = metricId};

            if (response.IsSuccessStatusCode)
            {
                var status = await response.Content.ReadAsStringAsync();
                dynamic currentObject = JObject.Parse(status);
                metricResult.Start = currentObject.value.start;
                metricResult.End = currentObject.value.end;
                metricResult.Value = currentObject.value.ToString();
            }

            return metricResult;
        }
    }
}