﻿using System;
using System.Diagnostics;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Storage.Queue;
using NTK2019.Infrastructure;

namespace NTK2019.CreatingCustomMetricsQueue
{
    internal class Program
    {
        private static PerformanceCounter totalOperations;
        private static PerformanceCounter operationsPerSecond;
        
        private const string PerfCounterCategoryName = "NTK2019";

        public static async Task Main(string[] args)
        {
            var storage = new Storage();
            var currentQueue = await storage.InitStorageAsync("ntk");

            CreatePerformanceCountersCategory();
            CreatePerformanceCounters();
            
            while (true)
            {
                await currentQueue.FetchAttributesAsync();

                int? cachedMessageCount = currentQueue.ApproximateMessageCount;
                Console.WriteLine("Število sporočil v vrsti: {0}", cachedMessageCount);

                if (cachedMessageCount.HasValue)
                {
                    foreach (CloudQueueMessage msg in await currentQueue.GetMessagesAsync(32,
                        TimeSpan.FromSeconds(5), null, null))
                    {
                        Console.WriteLine("Procesiranje sporočila: {0}", msg.AsString);
                        
                        totalOperations.Increment();
                        operationsPerSecond.Increment();
                        
                        await currentQueue.DeleteMessageAsync(msg);
                    }
                }

                await Task.Delay(2000);
            }
        }

        private static void CreatePerformanceCountersCategory()
        {
            try
            {
                if (!PerformanceCounterCategory.Exists(
                    PerfCounterCategoryName))
                {
                    var counters = new CounterCreationDataCollection();

                    var totalOps = new CounterCreationData
                    {
                        CounterName = "st dogodkov",
                        CounterHelp = "Stevilo sprocesiranih dogodkov",
                        CounterType = PerformanceCounterType.NumberOfItems32
                    };
                    counters.Add(totalOps);

                    //        PerformanceCounterType.Rate
                    var opsPerSecond = new CounterCreationData
                    {
                        CounterName = "st dogodkov na sekundo",
                        CounterHelp = "stevilo dogodkov na sekundo",
                        CounterType = PerformanceCounterType.RateOfCountsPerSecond32
                    };
                    counters.Add(opsPerSecond);

                    PerformanceCounterCategory.Create(PerfCounterCategoryName,
                        "Števci za Azure",
                        PerformanceCounterCategoryType.SingleInstance,
                        counters);
                }
            }
            catch (Exception exp)
            {
                Trace.TraceError("Napaka pri ustvarjanju perf counterjev " + exp);
            }
        }

        private static void CreatePerformanceCounters()
        {
            try
            {
                totalOperations = new PerformanceCounter
                {
                    CategoryName = PerfCounterCategoryName,
                    CounterName = "st dogodkov",
                    MachineName = ".",
                    ReadOnly = false
                };

                operationsPerSecond = new PerformanceCounter
                {
                    CategoryName = PerfCounterCategoryName,
                    CounterName = "st dogodkov na sekundo",
                    MachineName = ".",
                    ReadOnly = false
                };
            }
            catch (Exception exp)
            {
                Trace.TraceError("Napaka pri ustvarjanju števcev " + exp);
            }
        }
    }
}