using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.ApplicationInsights;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Azure.EventGrid.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using NTK2019.Web.Hub;
using NTK2019.Web.Models;

namespace NTK2019.Web.Controllers
{
    [Route("signal")]
    public class SignalrController : Controller
    {
        private readonly TelemetryClient telemetryClient = new TelemetryClient();
        
        private readonly IHubContext<Notification> hubcontext;

        public SignalrController(IHubContext<Notification> hubcontext)
        {
            this.hubcontext = hubcontext;
        }
        
        public IActionResult Index() => View();

        [HttpPost]
        [Route("messages")]
        public async Task<IActionResult> MessageWebHook()
        {
            telemetryClient.TrackEvent("AppInfo",
                new Dictionary<string, string>
                {
                { "Executed", DateTime.Now.ToShortDateString() },
                { "Method", "SignalrController > MessageWebHook > Receiving message" },
                { "Type", "POST" }
            });

            telemetryClient.Flush();
            
            var bodyStream = new StreamReader(HttpContext.Request.Body);
            var receivedEvent = bodyStream.ReadToEnd();

            if (HttpContext.Request.Headers.TryGetValue("Aeg-Event-Type", out var headerValues))
            {
                var validationHeaderValue = headerValues.FirstOrDefault();
                if (validationHeaderValue != null && validationHeaderValue.Contains("SubscriptionValidation"))
                {
                    var events = JsonConvert.DeserializeObject<EventGridEvent[]>(receivedEvent);
                    var code = events[0].Data as JObject;
                    var eventData = code?.ToObject<Models.SubscriptionValidationEventData>();
                    var responseData =
                        new SubscriptionValidationResponseData { ValidationResponse = eventData?.ValidationCode };
                    return Ok(JsonConvert.SerializeObject(responseData));
                }
            }

            var eventGridEvents = JsonConvert.DeserializeObject<EventGridEvent[]>(receivedEvent);
            
            telemetryClient.TrackEvent("AppInfo",
                new Dictionary<string, string>
                {
                    { "Executed", DateTime.Now.ToShortDateString() },
                    { "Method", "SignalrController > MessageWebHook > Processing events" },
                    { "Type", "POST" }
                });
            
            foreach (var eventGridEvent in eventGridEvents)
            {
                var dataObject = eventGridEvent.Data as JObject;

                if (string.Equals(eventGridEvent.EventType, "Microsoft.EventGrid.SubscriptionValidationEvent",
                                  StringComparison.OrdinalIgnoreCase))
                {
                    var eventData = dataObject?.ToObject<Models.SubscriptionValidationEventData>();

                    var responseData =
                        new SubscriptionValidationResponseData { ValidationResponse = eventData?.ValidationCode };
                    return Ok(responseData);
                }

                string message = $"{eventGridEvent.Subject} - pridobljen dne {DateTime.Now}";
                await hubcontext.Clients.All.SendAsync("broadcastMessage", message);
            }
            
            telemetryClient.Flush();

            return Ok();
        }
    }
}