﻿using Microsoft.ApplicationInsights.Extensibility;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Infrastructure;
using Microsoft.AspNetCore.Mvc.ViewFeatures;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using NTK2019.Web.Hub;
using NTK2019.Web.Settings;

namespace NTK2019.Web
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {            
            var signalrConfigurationSection = Configuration.GetSection("SignalrOptions");
            services.Configure<SignalrOptions>(signalrConfigurationSection);
            
            services.AddSingleton<IActionContextAccessor, ActionContextAccessor>();
            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            services.AddSingleton<ITempDataProvider, CookieTempDataProvider>();

            var signalR = signalrConfigurationSection.Get<SignalrOptions>();

            services
                .AddSignalR()
                .AddAzureSignalR(signalR.ConnectionString);
            
            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_2);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                TelemetryConfiguration.Active.TelemetryChannel.DeveloperMode = true;
                app.UseDeveloperExceptionPage();
            }
            else
            {
                //FOR PRODUCTION
                TelemetryConfiguration.Active.TelemetryChannel.DeveloperMode = false;
                app.UseExceptionHandler("/Home/Error");
            }

            app.UseStaticFiles();
            app.UseAzureSignalR(routes => routes.MapHub<Notification>("/notification"));

            app.UseMvc(routes =>
            {
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}
