﻿using System;
using System.IO;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.Http;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System.Collections.Generic;
using Ntk2019.AlertFunctions.Models;

namespace Ntk2019.AlertFunctions
{
    public static class AlertReaction
    {
        [FunctionName("AlertReaction")]
        public static async Task<IActionResult> Run(
            [HttpTrigger(AuthorizationLevel.Anonymous, "post", Route = null)] HttpRequest req,
            ILogger log)
        {
            log.LogInformation("Getting data from request.");

            string requestBody = await new StreamReader(req.Body).ReadToEndAsync();
            dynamic data = JsonConvert.DeserializeObject(requestBody);

            //notify back to user
            try
            {
                string TopicEndpoint = Environment.GetEnvironmentVariable("TopicEndpoint", EnvironmentVariableTarget.Process);
                log.LogInformation("Topic Endpoint" + TopicEndpoint);
                string Key = Environment.GetEnvironmentVariable("TopicEndpointKey", EnvironmentVariableTarget.Process);
                log.LogInformation("Topic key" + Key);

                var list = new List<Event>();
                var currentData = new Event() { EventTypes = EventTypes.GENERAL_INFORMATION, Subject = "Poslano preko Azure Function - " + data.data.context.name };
                list.Add(currentData);
                var report = JsonConvert.SerializeObject(list);
                log.LogInformation("Pošiljanje " + report);

                await FunctionHelper.PostMessageAsync(report, TopicEndpoint, Key, log);

                log.LogInformation("Sporočilo poslano " + DateTime.Now);
            }
            catch (Exception err)
            {
                log.LogError(err.Message);
            }

            return new OkObjectResult($"Posredovano {data.data.context.name} na subscription {data.data.context.subscriptionId}");
        }
    }
}
