﻿using System;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;

namespace Ntk2019.AlertFunctions
{
    public static class FunctionHelper
    {
        public static string GetEnvironmentVariable(string name) =>
            Environment.GetEnvironmentVariable(name, EnvironmentVariableTarget.Process);

        public static async Task<string> PostMessageAsync(string messageBody, string topicEndpoint, string key, ILogger log)
        {
            log.LogInformation("Sending message to " + topicEndpoint);
            var content = new StringContent(messageBody);
            using (var httpClient = new HttpClient())
            {
                var request = new HttpRequestMessage
                {
                    RequestUri = new Uri(topicEndpoint),
                    Method = HttpMethod.Post,
                    Content = content
                };

                request.Headers.Add("aeg-sas-key", key);

                var response = await httpClient.SendAsync(request);
                log.LogInformation("Message has been sent. Awaiting result - status code: " + response.IsSuccessStatusCode);
                if (!response.IsSuccessStatusCode)
                {
                    var error = await response.Content.ReadAsStringAsync();
                    log.LogInformation(error);
                    return error;
                }
            }

            return string.Empty;
        }
    }
}